class Node():
	"""docstring for ClassName"""
	def __init__(self,val=None,args=None):
		# Basic Attributes
		self.left = None
		self.right = None
		self.val = val
		# Advanced Attributes
		#self.name = None
